import matplotlib.pyplot as plt
import sqlite3

from figure import latexify, plot_cdf

def get_ratios(db_fname, sp_name, mp_name):
    conn = sqlite3.connect(db_fname)
    c = conn.cursor()

    configs = c.execute("SELECT DISTINCT bw_a_up, bw_a_down, bw_b_up, bw_b_down, delay_ms_a_up, delay_ms_a_down, delay_ms_b_up, delay_ms_b_down FROM results").fetchall()
    # One liner to remove tuple
    fs = [x[0] for x in c.execute("SELECT DISTINCT file_size FROM results").fetchall()][0]

    dct_sps = {}
    dct_mps = {}

    for config in configs:
        # Always take the best run
        dct_sp = c.execute("SELECT MIN(elapsed_time) FROM results WHERE bw_a_up = ? AND bw_a_down = ? AND bw_b_up = ? AND bw_b_down = ? AND delay_ms_a_up = ? AND delay_ms_a_down = ? AND delay_ms_b_up = ? AND delay_ms_b_down = ? AND file_size = ? AND test_name = ?",
            (config[0], config[1], config[2], config[3], config[4], config[5], config[6], config[7], fs, sp_name)).fetchone()[0]
        dct_mp = c.execute("SELECT MIN(elapsed_time) FROM results WHERE bw_a_up = ? AND bw_a_down = ? AND bw_b_up = ? AND bw_b_down = ? AND delay_ms_a_up = ? AND delay_ms_a_down = ? AND delay_ms_b_up = ? AND delay_ms_b_down = ? AND file_size = ? AND test_name = ?",
            (config[0], config[1], config[2], config[3], config[4], config[5], config[6], config[7], fs, mp_name)).fetchone()[0]
        if dct_mp == 0 or dct_sp == 0:
            continue

        dct_sps[config] = dct_sp
        dct_mps[config] = dct_mp

    return dct_sps, dct_mps, configs

dct_sps_pquic, dct_mps_pquic, configs = get_ratios('mpiquic_asym/results_mp_asym_cond.db', 'sp_quic', 'mp_quic')
dct_sps_pquic_rev, dct_mps_pquic_rev, configs_rev = get_ratios('mpiquic_asym/results_mp_asym_cond_rev.db', 'sp_quic', 'mp_quic')
dct_sps_pquic.update(dct_sps_pquic_rev)
dct_mps_pquic.update(dct_mps_pquic)
configs = configs + configs_rev

dct_sps_quic_go, dct_mps_quic_go, _ = get_ratios('mpiquic_asym/results_quic_go.db', 'sp_quic_go', 'mp_quic_go')
dct_sps_quic_go_rev, dct_mps_quic_go_rev, _ = get_ratios('mpiquic_asym/results_quic_go_rev.db', 'sp_quic_go', 'mp_quic_go')
dct_sps_quic_go.update(dct_sps_quic_go_rev)
dct_mps_quic_go.update(dct_mps_quic_go_rev)

dct_sps_mptcp, dct_mps_mptcp, _ = get_ratios('mptcp_asym/results_mptcp_asym.db', 'tcp', 'mptcp')
dct_sps_mptcp_rev, dct_mps_mptcp_rev, _ = get_ratios('mptcp_asym/results_mptcp_asym_rev.db', 'tcp', 'mptcp')
dct_sps_mptcp.update(dct_sps_mptcp_rev)
dct_mps_mptcp.update(dct_mps_mptcp_rev)

latexify()

mp_ratios = []
print("Multipath")
for config in dct_mps_pquic.keys():
    pq = dct_mps_pquic[config]
    mp = dct_mps_mptcp[config]
    ratio = pq / mp
    if ratio >= 1.05:
        print(ratio, pq, mp, config)
    mp_ratios.append(ratio)

sp_ratios = []
print("Single path")
for config in dct_sps_pquic.keys():
    pq = dct_sps_pquic[config]
    mp = dct_sps_mptcp[config]
    ratio = pq / mp
    if ratio >= 1.05:
        print(ratio, pq, mp, config)
    sp_ratios.append(ratio)


plt.figure()
plt.clf()

f, (ax1, ax2) = plt.subplots(1, 2, sharey=True)

plot_cdf(ax1, sp_ratios, label="One Path", color="#bdbdbd", ls="--") # b2abd2
plot_cdf(ax1, mp_ratios, label="Two Paths", color="#636363", ls="-") # e66101
ax1.set_xscale("log")
ax1.set_xlim(xmin=0.5, xmax=2.0)
ax1.set_ylim(ymin=0, ymax=1)
ax1.set_ylabel("CDF")
ax1.set_xlabel(r'$\frac{\textsc{DCT MFQUIC}}{\textsc{DCT MPTCP}}$')
from matplotlib.ticker import (NullFormatter)
ax1.xaxis.set_minor_formatter(NullFormatter())
ax1.set_xticks([0.5,  1, 2])
ax1.grid()
ax1.set_xticklabels([r'$0.5$', r'$1$', r'$2$'])
lgd = ax1.legend(loc='upper center', bbox_to_anchor=(1.1, 1.25),
          ncol=2)

mp_ratios = []
print("Multipath")
for config in dct_mps_pquic.keys():
    pq = dct_mps_pquic[config]
    qg = dct_mps_quic_go[config]
    ratio = pq / qg
    if ratio >= 1.05:
        print(ratio, pq, qg, config)
    mp_ratios.append(ratio)

sp_ratios = []
print("Single path")
for config in dct_sps_pquic.keys():
    pq = dct_sps_pquic[config]
    qg = dct_sps_quic_go[config]
    ratio = pq / qg
    if ratio >= 1.05:
        print(ratio, pq, qg, config)
    sp_ratios.append(ratio)

plot_cdf(ax2, sp_ratios, label="One Path", color="#bdbdbd", ls="--") # b2abd2
plot_cdf(ax2, mp_ratios, label="Two Paths", color="#636363", ls="-") # e66101

ax2.set_xscale("log")
ax2.set_xlim(xmin=0.5, xmax=2.0)
ax2.set_xticks([0.5, 1, 2])
ax2.xaxis.set_minor_formatter(NullFormatter())
ax2.set_xticklabels([r'$0.5$', r'$1$', r'$2$'])
ax2.set_xlabel(r'$\frac{\textsc{DCT MFQUIC}}{\textsc{DCT MPQUIC}}$')
ax2.grid()

plt.savefig("mpiquic_asym_mp_diff_cdf_both.pgf", bbox_extra_artists=(lgd,), bbox_inches='tight')