import matplotlib.pyplot as plt
import numpy as np
import sqlite3

from figure import latexify, plot_cdf

def get_ratios(db_fname, sp_name, mp_name):
    conn = sqlite3.connect(db_fname)
    c = conn.cursor()

    configs = c.execute("SELECT DISTINCT bw_a_up, bw_a_down, bw_b_up, bw_b_down, delay_ms_a_up, delay_ms_a_down, delay_ms_b_up, delay_ms_b_down FROM results").fetchall()
    # One liner to remove tuple
    fs = [x[0] for x in c.execute("SELECT DISTINCT file_size FROM results").fetchall()][0]

    dct_ratios = []
    for config in configs:
        # Always take the best run
        dct_sp = c.execute("SELECT MIN(elapsed_time) FROM results WHERE bw_a_up = ? AND bw_a_down = ? AND bw_b_up = ? AND bw_b_down = ? AND delay_ms_a_up = ? AND delay_ms_a_down = ? AND delay_ms_b_up = ? AND delay_ms_b_down = ? AND file_size = ? AND test_name = ?",
            (config[0], config[1], config[2], config[3], config[4], config[5], config[6], config[7], fs, sp_name)).fetchone()[0]
        dct_mp = c.execute("SELECT MIN(elapsed_time) FROM results WHERE bw_a_up = ? AND bw_a_down = ? AND bw_b_up = ? AND bw_b_down = ? AND delay_ms_a_up = ? AND delay_ms_a_down = ? AND delay_ms_b_up = ? AND delay_ms_b_down = ? AND file_size = ? AND test_name = ?",
            (config[0], config[1], config[2], config[3], config[4], config[5], config[6], config[7], fs, mp_name)).fetchone()[0]
        if dct_mp == 0 or dct_sp == 0:
            continue

        dct_ratio = dct_mp / dct_sp
        if dct_ratio >= 1.0 and fs >= 1000000:
            print(config, dct_ratio)
        dct_ratios.append(dct_ratio)

    return dct_ratios, configs

dct_ratios_pquic, configs = get_ratios('mpiquic_asym/results_mp_asym_cond.db', 'sp_quic', 'mp_quic')
dct_ratios_pquic_rev, configs_rev = get_ratios('mpiquic_asym/results_mp_asym_cond_rev.db', 'sp_quic', 'mp_quic')
dct_ratios_pquic = dct_ratios_pquic + dct_ratios_pquic_rev
configs = configs + configs_rev

dct_ratios_quic_go, _ = get_ratios('mpiquic_asym/results_quic_go.db', 'sp_quic_go', 'mp_quic_go')
dct_ratios_quic_go_rev, _ = get_ratios('mpiquic_asym/results_quic_go_rev.db', 'sp_quic_go', 'mp_quic_go')
dct_ratios_quic_go = dct_ratios_quic_go + dct_ratios_quic_go_rev

dct_ratios_mptcp, _ = get_ratios('mptcp_asym/results_mptcp_asym.db', 'tcp', 'mptcp')
dct_ratios_mptcp_rev, _ = get_ratios('mptcp_asym/results_mptcp_asym_rev.db', 'tcp', 'mptcp')
dct_ratios_mptcp = dct_ratios_mptcp + dct_ratios_mptcp_rev

print(len(dct_ratios_pquic), len(dct_ratios_quic_go), len(dct_ratios_mptcp), len(configs))

latexify()

plot_cdf(plt, dct_ratios_mptcp, label="\\textsc{MPTCP}", color="#9d9d9d", ls=":") # bdbdbd
plot_cdf(plt, dct_ratios_quic_go, label="\\textsc{MPQUIC}", color="#998ec3", ls="--") # bdbdbd
plot_cdf(plt, dct_ratios_pquic, label="\\textsc{MFQUIC}", color="#f1a340", ls="-") # 636363

print("Median MPTCP is {}, median quic-go is {} and median picoquic is {}".format(np.median(dct_ratios_mptcp), np.median(dct_ratios_quic_go), np.median(dct_ratios_pquic)))

plt.xlabel("DCT Two Paths / DCT One Path")
plt.xscale("log")
plt.xlim(xmin=0.1, xmax=10)
plt.ylim(ymin=0, ymax=1)
plt.ylabel("CDF")
plt.grid()
plt.legend(loc="best")
plt.tight_layout()
plt.savefig("mpiquic_asym_ratio_cdf.pgf")